(if (not (featurep 'liece-setup)) (progn

;;;### (autoloads (liece-nick-redisplay-buffer liece-nick-update-region liece-nick-update liece-nick-change liece-nick-part liece-nick-join liece-command-toggle-nick-buffer-mode) "liece-nick" "lisp/liece-nick.el")

(autoload 'liece-command-toggle-nick-buffer-mode "liece-nick" nil t nil)

(autoload 'liece-nick-join "liece-nick" nil nil nil)

(autoload 'liece-nick-part "liece-nick" nil nil nil)

(autoload 'liece-nick-change "liece-nick" nil nil nil)

(autoload 'liece-nick-update "liece-nick" nil nil nil)

(autoload 'liece-nick-update-region "liece-nick" nil nil nil)

(autoload 'liece-nick-redisplay-buffer "liece-nick" nil nil nil)

;;;***

;;;### (autoloads (liece-channel-list-redisplay-buffer liece-channel-redisplay-buffer liece-command-toggle-channel-buffer-mode) "liece-channel" "lisp/liece-channel.el")

(autoload 'liece-command-toggle-channel-buffer-mode "liece-channel" "\
Toggle visibility of channel buffer." t nil)

(autoload 'liece-channel-redisplay-buffer "liece-channel" nil nil nil)

(autoload 'liece-channel-list-redisplay-buffer "liece-channel" nil nil nil)

;;;***

;;;### (autoloads (liece-display-message) "liece-message" "lisp/liece-message.el")

(autoload 'liece-display-message "liece-message" nil nil nil)

;;;***

;;;### (autoloads (liece-intl-get-msgstr) "liece-intl" "lisp/liece-intl.el")

(autoload 'liece-intl-get-msgstr "liece-intl" nil nil 'macro)

(defalias '_ 'liece-intl-get-msgstr)

;;;***

;;;### (autoloads (liece-open-network-stream) "liece-tcp" "lisp/liece-tcp.el")

(autoload 'liece-open-network-stream "liece-tcp" nil nil nil)

;;;***

;;;### (autoloads (liece-configure-windows liece-command-set-window-style) "liece-window" "lisp/liece-window.el")

(autoload 'liece-command-set-window-style "liece-window" nil t nil)

(autoload 'liece-configure-windows "liece-window" nil nil nil)

;;;***

;;;### (autoloads (liece-insert liece-replace liece-clear-system liece-nick-mode liece-channel-list-mode liece-channel-mode liece-others-mode liece-dialogue-mode liece-command-mode liece-close-server liece-server-opened) "liece" "lisp/liece.el")

(autoload 'liece-server-opened "liece" "\
Return server process status.
Return non-nil if stream is opened." nil 'macro)

(autoload 'liece-close-server "liece" "\
Close chat server." nil nil)

(autoload 'liece-command-mode "liece" "\
Major mode for Liece.  Normal edit function are available.
Typing Return or Linefeed enters the current line in the dialogue.
The following special commands are available:
For a list of the generic commands type \\[liece-command-generic] ? RET.
\\{liece-command-mode-map}" t nil)

(autoload 'liece-dialogue-mode "liece" "\
Major mode for displaying the IRC dialogue.
All normal editing commands are turned off.
Instead, these commands are available:
\\{liece-dialogue-mode-map}" nil nil)

(autoload 'liece-others-mode "liece" "Major mode for displaying the IRC others message except current channel.\nAll normal editing commands are turned off.\nInstead, these commands are available:\n\\{liece-others-mode-map}" t nil)

(autoload 'liece-channel-mode "liece" "Major mode for displaying the IRC current channel buffer.\nAll normal editing commands are turned off.\nInstead, these commands are available:\n\\{liece-channel-mode-map}" t nil)

(autoload 'liece-channel-list-mode "liece" "\
Major mode for displaying channel list.
All normal editing commands are turned off." nil nil)

(autoload 'liece-nick-mode "liece" "\
Major mode for displaying members in the IRC current channel buffer.
All normal editing commands are turned off.
Instead, these commands are available:
\\{liece-nick-mode-map}" nil nil)

(autoload 'liece-clear-system "liece" "\
Clear all Liece variables and buffers." t nil)

(autoload 'liece-replace "liece" "\
Replace in buffer or list of buffers BUFFER with matching MATCH.
Argument DEFSTRING used when no matches are there.
Argument OLDSTRING is replaced with NEWSTRING." nil nil)

(autoload 'liece-insert "liece" "\
Insert before point of BUFFER STRING with decorating." nil nil)

;;;***
(provide 'liece-setup)))
