;;; DO NOT MODIFY THIS FILE
(if (featurep 'liece-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "lisp/_pkg.el")

(package-provide 'liece :version 1.1 :type 'regular)

;;;***

;;;### (autoloads (liece) "liece" "lisp/liece.el")

(autoload 'liece "liece" "\
Connect to the IRC server and start chatting.
If optional argument CONFIRM is non-nil, ask which IRC server to connect.
If already connected, just pop up the windows." t nil)

;;;***

(provide 'liece-autoloads)
